"""This script tests the run cli command"""

import logging

import pytest
from _pytest.fixtures import FixtureRequest
from _pytest.logging import LogCaptureFixture
from click.testing import CliRunner

from cg.cli.workflow.base import workflow as workflow_cli
from cg.constants import EXIT_SUCCESS, Workflow
from cg.constants.nextflow import NEXTFLOW_WORKFLOWS
from cg.models.cg_config import CGConfig


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_without_options(cli_runner: CliRunner, workflow: Workflow, request: FixtureRequest):
    """Test run command for workflow without options."""
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN no case id

    # WHEN invoking the command without additional parameters
    result = cli_runner.invoke(workflow_cli, [workflow, "run"], obj=context)

    # THEN command should not exit successfully
    assert result.exit_code != EXIT_SUCCESS

    # THEN command log should inform about missing arguments
    assert "Missing argument" in result.output


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_with_missing_case(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    case_id_does_not_exist: str,
    request: FixtureRequest,
):
    """Test run command for workflow with a missing case."""
    caplog.set_level(logging.ERROR)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case not in the StatusDB database
    assert not context.status_db.get_case_by_internal_id(internal_id=case_id_does_not_exist)

    # WHEN running
    result = cli_runner.invoke(workflow_cli, [workflow, "run", case_id_does_not_exist], obj=context)

    # THEN command should not exit successfully
    assert result.exit_code != EXIT_SUCCESS

    # THEN the error log should indicate that the case is invalid
    assert case_id_does_not_exist in caplog.text
    assert "could not be found" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_case_without_samples(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    no_sample_case_id: str,
    request: FixtureRequest,
):
    """Test command with a case lacking linked samples."""
    caplog.set_level(logging.ERROR)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case without linked samples

    # WHEN invoking the command
    result = cli_runner.invoke(
        workflow_cli, [workflow, "run", no_sample_case_id, "--dry-run"], obj=context
    )

    # THEN command should not exit successfully
    assert result.exit_code != EXIT_SUCCESS

    # THEN warning should be printed that no sample is found
    assert no_sample_case_id in caplog.text
    assert "no samples" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_case_without_config_files(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    request: FixtureRequest,
):
    """Test command with case_id and no config file."""
    caplog.set_level(logging.ERROR)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # WHEN invoking a run command
    result = cli_runner.invoke(workflow_cli, [workflow, "run", case_id], obj=context)

    # THEN command should not exit successfully
    assert result.exit_code != EXIT_SUCCESS

    # THEN the error log should indicate that the config file is missing
    assert "No config file found" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_case_from_start_dry_run(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    request: FixtureRequest,
):
    """Test dry-run for a case with existing config files."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN mocked config files
    request.getfixturevalue(f"{workflow}_mock_config")

    # WHEN invoking a command with dry-run specified
    result = cli_runner.invoke(
        workflow_cli, [workflow, "run", case_id, "--from-start", "--dry-run"], obj=context
    )

    # THEN command should execute successfully
    assert result.exit_code == EXIT_SUCCESS

    # THEN command should use tower
    assert "using Tower" in caplog.text
    assert "path/to/bin/tw launch" in caplog.text
    assert "--work-dir" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_run_case_with_revision_dry_run(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    request: FixtureRequest,
):
    """Test dry-run for a case with existing config files with a given revision."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN a mocked config
    request.getfixturevalue(f"{workflow}_mock_config")

    # WHEN invoking a command with dry-run and revision specified
    result = cli_runner.invoke(
        workflow_cli,
        [workflow, "run", case_id, "--dry-run", "--from-start", "--revision", "2.1.0"],
        obj=context,
    )

    # THEN command should execute successfully
    assert result.exit_code == EXIT_SUCCESS

    # THEN command should use the specified revision
    assert "--revision 2.1.0" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_resume_case_dry_run(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    tower_id,
    request: FixtureRequest,
):
    """Test dry-run to resume a case with existing config fjles."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN a mocked config
    # request.getfixturevalue(f"{workflow}_mock_config")

    # WHEN invoking a command with dry-run and nf-tower-id specified
    result = cli_runner.invoke(
        workflow_cli,
        [workflow, "run", case_id, "--nf-tower-id", tower_id, "--dry-run"],
        obj=context,
    )

    # THEN command should execute successfully
    assert result.exit_code == EXIT_SUCCESS

    # THEN command should use tower for relaunch
    assert "Workflow will be resumed from run" in caplog.text
    assert "tw runs relaunch" in caplog.text


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_resume_case_with_missing_tower_id(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    raredisease_mock_config,
    request: FixtureRequest,
):
    """Test resume command without providing NF-Tower ID and without existing Trailblazer config file."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN a mocked config

    # WHEN invoking a command with dry-run specified
    cli_runner.invoke(workflow_cli, [workflow, "run", case_id, "--dry-run"], obj=context)

    # THEN command should raise error
    assert "Could not resume analysis: No NF-Tower ID found for case" in caplog.text
    pytest.raises(FileNotFoundError)


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_resume_using_nextflow_dry_run(
    cli_runner: CliRunner,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    raredisease_mock_config,
    request: FixtureRequest,
):
    """Test command with case_id and config file using nextflow."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN a mocked config

    # WHEN invoking a command with dry-run specified
    result = cli_runner.invoke(
        workflow_cli, [workflow, "run", case_id, "--dry-run", "--use-nextflow"], obj=context
    )

    # THEN command should execute successfully
    assert result.exit_code == EXIT_SUCCESS

    # THEN command should use nextflow
    assert "using Nextflow" in caplog.text
    assert "path/to/bin/nextflow" in caplog.text
    assert "-work-dir" in caplog.text

    # THEN command should include resume flag
    assert "-resume" in caplog.text



"""Tests report data validation"""

from cg.constants.report import NA_FIELD
from cg.meta.report.field_validators import (
    get_empty_report_data,
    get_million_read_pairs,
    get_missing_report_data,
)


def test_get_empty_report_data(report_api_mip_dna, case_mip_dna):
    """Tests the empty report fields retrieval."""

    # GIVEN a report data model
    report_data = report_api_mip_dna.get_report_data(
        case_mip_dna.internal_id, case_mip_dna.analyses[0].started_at
    )

    # GIVEN some empty fields
    report_data.version = None
    report_data.accredited = None
    report_data.customer.id = NA_FIELD
    report_data.customer.scout_access = False
    report_data.case.samples[0].methods.library_prep = ""
    report_data.case.samples[0].metadata.million_read_pairs = None
    report_data.case.samples[0].metadata.duplicates = None
    report_data.case.samples[1].metadata.duplicates = None

    # WHEN retrieving the missing data
    empty_fields = get_empty_report_data(report_data)

    # THEN assert that the empty fields are correctly retrieved
    assert "version" in empty_fields["report"]
    assert "accredited" in empty_fields["report"]
    assert "id" in empty_fields["customer"]
    assert "scout_access" not in empty_fields["customer"]
    assert "library_prep" in empty_fields["methods"]["ADM1"]
    assert "million_read_pairs" in empty_fields["metadata"]["ADM1"]
    assert "duplicates" in empty_fields["metadata"]["ADM1"]
    assert "duplicates" in empty_fields["metadata"]["ADM2"]


def test_get_missing_report_data(report_api_mip_dna, case_mip_dna):
    """Checks the missing report fields retrieval."""

    # GIVEN a report data model
    report_data = report_api_mip_dna.get_report_data(
        case_mip_dna.internal_id, case_mip_dna.analyses[0].started_at
    )
    report_data.case.samples[0].application.prep_category = "wgs"  # ADM1 sample (WGS)
    report_data.case.samples[1].application.prep_category = "wes"  # ADM2 sample (WES)

    # GIVEN a dictionary of report empty fields and a list of required MIP DNA report fields
    empty_fields = {
        "report": ["version", "accredited"],
        "customer": ["id"],
        "methods": {"ADM1": ["library_prep"]},
        "metadata": {
            "ADM1": ["bait_set", "million_read_pairs", "duplicates"],
            "ADM2": ["bait_set", "duplicates"],
        },
    }

    required_fields = report_api_mip_dna.get_required_fields(report_data.case)

    # WHEN retrieving the missing data
    missing_fields = get_missing_report_data(empty_fields, required_fields)

    # THEN assert that the required fields are identified
    assert "version" not in missing_fields["report"]
    assert "accredited" in missing_fields["report"]
    assert "customer" not in missing_fields
    assert "methods" not in missing_fields
    assert "million_read_pairs" in missing_fields["metadata"]["ADM1"]
    assert "duplicates" in missing_fields["metadata"]["ADM1"]
    assert "duplicates" in missing_fields["metadata"]["ADM2"]
    assert "bait_set" not in missing_fields["metadata"]["ADM1"]
    assert "bait_set" in missing_fields["metadata"]["ADM2"]
    assert "ADM3" not in missing_fields["metadata"]


def test_get_million_read_pairs():
    """Tests millions read pairs computation."""

    # GIVEN a number os sequencing reads and its representation in millions of read pairs
    sample_reads = 1_200_000_000
    expected_million_read_pairs = 600.0

    # WHEN obtaining the number of reds in millions of read pairs
    million_read_pairs = get_million_read_pairs(sample_reads)

    # THEN the expected value should match the calculated one
    assert million_read_pairs == expected_million_read_pairs


def test_get_million_read_pairs_zero_input():
    """Tests millions read pairs computation when the sample reads number is zero."""

    # GIVEN zero as number of reads
    sample_reads = 0

    # WHEN retrieving the number of reds in millions of read pairs
    million_read_pairs = get_million_read_pairs(sample_reads)

    # THEN the obtained value should be zero
    assert million_read_pairs == 0.0



import logging

import pytest
from _pytest.fixtures import FixtureRequest
from _pytest.logging import LogCaptureFixture
from click.testing import CliRunner

from cg.apps.hermes.hermes_api import HermesApi
from cg.apps.hermes.models import CGDeliverables
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.cli.workflow.base import workflow as workflow_cli
from cg.constants import EXIT_FAIL, EXIT_SUCCESS, Workflow
from cg.constants.constants import CaseActions
from cg.constants.nextflow import NEXTFLOW_WORKFLOWS
from cg.meta.workflow.nf_analysis import NfAnalysisAPI
from cg.models.cg_config import CGConfig


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_store_success(
    cli_runner: CliRunner,
    real_housekeeper_api: HousekeeperAPI,
    workflow: Workflow,
    caplog: LogCaptureFixture,
    deliverables_template_content: list[dict],
    request: FixtureRequest,
    mocker,
):
    """Test to ensure all parts of store command are run successfully given ideal conditions."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case for which we mocked files created after a successful run
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")
    hermes_deliverables: dict = request.getfixturevalue(f"{workflow}_hermes_deliverables")
    request.getfixturevalue(f"{workflow}_mock_deliverable_dir")
    request.getfixturevalue(f"{workflow}_mock_analysis_finish")
    deliverables_response_data = request.getfixturevalue(f"{workflow}_deliverables_response_data")

    # GIVEN a mocked deliverables template
    mocker.patch.object(
        NfAnalysisAPI,
        "get_deliverables_template_content",
        return_value=deliverables_template_content,
    )

    # GIVEN that the Housekeeper store is empty
    context.housekeeper_api_: HousekeeperAPI = real_housekeeper_api
    context.meta_apis["analysis_api"].housekeeper_api = real_housekeeper_api

    # GIVEN that the bundle is not present in Housekeeper
    assert not context.housekeeper_api.bundle(case_id)

    # GIVEN that the analysis is not already stored in status_db
    assert not context.status_db.get_case_by_internal_id(internal_id=case_id).analyses

    # GIVEN that HermesAPI returns a deliverables output
    mocker.patch.object(HermesApi, "convert_deliverables")
    HermesApi.convert_deliverables.return_value = CGDeliverables.model_validate(hermes_deliverables)

    # GIVEN Hermes parses deliverables and generates a valid response
    mocker.patch.object(HermesApi, "create_housekeeper_bundle")
    HermesApi.create_housekeeper_bundle.return_value = deliverables_response_data

    # WHEN storing the case files
    result = cli_runner.invoke(workflow_cli, [workflow, "store", case_id], obj=context)

    # THEN bundle should be successfully added to Housekeeper and StatusDB
    assert result.exit_code == EXIT_SUCCESS
    assert "Analysis successfully stored in Housekeeper" in caplog.text
    assert "Analysis successfully stored in StatusDB" in caplog.text
    assert context.status_db.get_case_by_internal_id(internal_id=case_id).analyses
    assert context.housekeeper_api.bundle(case_id)


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_store_fail(
    cli_runner: CliRunner,
    workflow: Workflow,
    real_housekeeper_api: HousekeeperAPI,
    deliverables_template_content: list[dict],
    caplog: LogCaptureFixture,
    request: FixtureRequest,
    mocker,
):
    """Test store command fails when a case did not finish for a workflow."""
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case id where analysis finish is not mocked
    case_id_fail: str = request.getfixturevalue(f"{workflow}_case_id")

    # GIVEN a mocked deliverables template
    mocker.patch.object(
        NfAnalysisAPI,
        "get_deliverables_template_content",
        return_value=deliverables_template_content,
    )

    # GIVEN that the Housekeeper store is empty
    context.housekeeper_api_: HousekeeperAPI = real_housekeeper_api
    context.meta_apis["analysis_api"].housekeeper_api = real_housekeeper_api

    # GIVEN that the bundle is not present in Housekeeper
    assert not context.housekeeper_api.bundle(case_id_fail)

    # WHEN running command
    result_fail = cli_runner.invoke(workflow_cli, [workflow, "store", case_id_fail], obj=context)

    # THEN bundle exist status should be
    assert result_fail.exit_code != EXIT_SUCCESS


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_store_available_success(
    cli_runner: CliRunner,
    workflow: Workflow,
    real_housekeeper_api: HousekeeperAPI,
    deliverables_template_content: list[dict],
    request: FixtureRequest,
    caplog: LogCaptureFixture,
    mocker,
):
    """
    Test to ensure all parts of the compound store-available command are executed given ideal
    conditions.
    """
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case for which we mocked files created after a successful run
    hermes_deliverables: dict = request.getfixturevalue(f"{workflow}_hermes_deliverables")
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")
    request.getfixturevalue(f"{workflow}_mock_deliverable_dir")
    request.getfixturevalue(f"{workflow}_mock_analysis_finish")

    # GIVEN that the Housekeeper store is empty
    context.housekeeper_api_: HousekeeperAPI = real_housekeeper_api
    context.meta_apis["analysis_api"].housekeeper_api = real_housekeeper_api

    # GIVEN that HermesAPI returns a deliverables output
    mocker.patch.object(HermesApi, "convert_deliverables")
    HermesApi.convert_deliverables.return_value = CGDeliverables.model_validate(hermes_deliverables)

    # GIVEN there is available cases to store
    context.status_db.get_case_by_internal_id(case_id).action = CaseActions.RUNNING
    context.status_db.session.commit()

    # WHEN running command
    result = cli_runner.invoke(workflow_cli, [workflow, "store-available"], obj=context)

    # THEN all expected cases are picked up for storing
    assert f"Storing deliverables for {case_id}" in caplog.text

    # THEN case has an associated analysis
    assert len(context.status_db.get_case_by_internal_id(case_id).analyses) == 1

    # THEN bundle can be found in Housekeeper
    assert context.housekeeper_api.bundle(case_id)

    # THEN StatusDB action is set to None
    assert context.status_db.get_case_by_internal_id(case_id).action is None

    # THEN command exits successfully
    assert result.exit_code == EXIT_SUCCESS


@pytest.mark.parametrize(
    "workflow",
    NEXTFLOW_WORKFLOWS,
)
def test_store_available_fail(
    cli_runner: CliRunner,
    workflow: Workflow,
    real_housekeeper_api: HousekeeperAPI,
    deliverables_template_content: list[dict],
    case_id_not_enough_reads: str,
    sample_id: str,
    request: FixtureRequest,
    caplog: LogCaptureFixture,
    mocker,
):
    """
    Test that store-available picks up eligible cases and does not stop after failing to store a case.
    """
    caplog.set_level(logging.INFO)
    context: CGConfig = request.getfixturevalue(f"{workflow}_context")

    # GIVEN a case for which we mocked files created after a successful run
    hermes_deliverables: dict = request.getfixturevalue(f"{workflow}_hermes_deliverables")
    case_id: str = request.getfixturevalue(f"{workflow}_case_id")
    request.getfixturevalue(f"{workflow}_mock_deliverable_dir")
    request.getfixturevalue(f"{workflow}_mock_analysis_finish")

    # GIVEN a case where analysis finish is not mocked
    failed_case_id: str = case_id_not_enough_reads

    # GIVEN that the Housekeeper store is empty
    context.housekeeper_api_: HousekeeperAPI = real_housekeeper_api
    context.meta_apis["analysis_api"].housekeeper_api = real_housekeeper_api

    # GIVEN that HermesAPI returns a deliverables output
    mocker.patch.object(HermesApi, "convert_deliverables")
    HermesApi.convert_deliverables.return_value = CGDeliverables.model_validate(hermes_deliverables)

    # GIVEN there is available cases to store
    context.status_db.get_case_by_internal_id(case_id).action = CaseActions.RUNNING
    context.status_db.get_case_by_internal_id(failed_case_id).action = CaseActions.RUNNING
    context.status_db.session.commit()

    # WHEN running command
    result = cli_runner.invoke(workflow_cli, [workflow, "store-available"], obj=context)

    # THEN command should not exit successfully
    assert result.exit_code == EXIT_FAIL

    # THEN all expected cases are picked up for storing
    for case in [failed_case_id, case_id]:
        assert f"Storing deliverables for {case}" in caplog.text

    # THEN one case failed to store
    assert f"Error storing {failed_case_id}" in caplog.text

    # THEN case failing store does not have an associated analysis
    assert not context.status_db.get_case_by_internal_id(failed_case_id).analyses

    # THEN case with successful store has an associated analysis
    assert len(context.status_db.get_case_by_internal_id(case_id).analyses) == 1

    # THEN case failing store does not have an associated Housekeeper bundle
    assert not context.housekeeper_api.bundle(failed_case_id)

    # THEN bundle can be found in Housekeeper for successful case
    assert context.housekeeper_api.bundle(case_id)

    # THEN StatusDB action remains as running for case failing store
    assert context.status_db.get_case_by_internal_id(failed_case_id).action == CaseActions.RUNNING

    # THEN StatusDB action is set to None for successful case
    assert not context.status_db.get_case_by_internal_id(case_id).action



"""Module for PacBio fixtures returning Path objects."""

from pathlib import Path

import pytest


# Directory fixtures
@pytest.fixture
def pac_bio_fixtures_dir(devices_dir: Path) -> Path:
    """Return the path to the PacBio fixtures directory."""
    return Path(devices_dir, "pacbio")


@pytest.fixture
def pac_bio_wrong_metrics_dir(pac_bio_fixtures_dir: Path) -> Path:
    """Return the path to the PacBio metrics directory."""
    return Path(pac_bio_fixtures_dir, "wrong_metrics")


@pytest.fixture
def pac_bio_runs_dir(pac_bio_fixtures_dir: Path) -> Path:
    """Return the path to the PacBio run directory."""
    return Path(pac_bio_fixtures_dir, "SMRTcells")


@pytest.fixture
def pac_bio_test_run_dir(pac_bio_runs_dir: Path, pac_bio_test_run_name: str) -> Path:
    """Return the path to a PacBio run directory."""
    return Path(pac_bio_runs_dir, pac_bio_test_run_name)


@pytest.fixture
def pac_bio_smrt_cell_dir_1_a01(pac_bio_test_run_dir: Path) -> Path:
    """Return the path to a PacBio SMRT cell directory."""
    return Path(pac_bio_test_run_dir, "1_A01")


@pytest.fixture
def pac_bio_run_statistics_dir(pac_bio_smrt_cell_dir_1_a01: Path) -> Path:
    """Return the path to the PacBio SMRT cell statistics directory."""
    return Path(pac_bio_smrt_cell_dir_1_a01, "statistics")


@pytest.fixture
def pac_bio_run_reports_dir(pac_bio_run_statistics_dir: Path) -> Path:
    """Return the path to the PacBio SMRT cell zipped_reports directory"""
    return Path(pac_bio_run_statistics_dir, "zipped_reports")


@pytest.fixture
def pac_bio_wrong_metrics_file(pac_bio_wrong_metrics_dir: Path) -> Path:
    """Return the path to a temporary PacBio statistics directory."""
    return Path(pac_bio_wrong_metrics_dir, "metrics.json")



"""Tests for the Illumina Sample Sequencing Metrics filters."""

from sqlalchemy.orm import Query

from cg.store.filters.status_illumina_metrics_filters import filter_by_lane
from cg.store.models import IlluminaSampleSequencingMetrics
from cg.store.store import Store
from tests.small_helpers import SmallHelpers


def test_filter_by_lane(
    store_with_illumina_sequencing_data: Store,
    seven_canonical_flow_cells_selected_sample_ids: list[list[str]],
    small_helpers: SmallHelpers,
):
    """Test to filter metrics by lane."""
    # GIVEN a store with Illumina Sample Sequencing Metrics for each sample in the run directories
    metrics: Query = store_with_illumina_sequencing_data._get_query(
        table=IlluminaSampleSequencingMetrics
    )
    number_of_samples_in_store: int = small_helpers.length_of_nested_list(
        seven_canonical_flow_cells_selected_sample_ids
    )
    assert metrics.count() == number_of_samples_in_store

    # GIVEN a lane number
    lane: int = 1

    # WHEN filtering metrics by lane
    filtered_metrics: Query = filter_by_lane(metrics=metrics, lane=lane)

    # THEN assert that the function returns a Query of metric objects with the expected lane
    assert filtered_metrics.count() < number_of_samples_in_store
    for metric in filtered_metrics.all():
        assert metric.flow_cell_lane == lane



import logging

import click

from cg.cli.utils import CLICK_CONTEXT_SETTINGS
from cg.cli.workflow.commands import (
    ARGUMENT_CASE_ID,
    OPTION_ANALYSIS_PARAMETERS_CONFIG,
    link,
    resolve_compression,
    store,
    store_available,
)
from cg.constants import EXIT_FAIL, EXIT_SUCCESS
from cg.constants.cli_options import DRY_RUN
from cg.exc import AnalysisNotReadyError, CgError
from cg.meta.workflow.analysis import AnalysisAPI
from cg.meta.workflow.mutant import MutantAnalysisAPI
from cg.models.cg_config import CGConfig

LOG = logging.getLogger(__name__)


@click.group(invoke_without_command=True, context_settings=CLICK_CONTEXT_SETTINGS)
@click.pass_context
def mutant(context: click.Context) -> None:
    """Mutant analysis workflow"""
    AnalysisAPI.get_help(context)
    context.obj.meta_apis["analysis_api"] = MutantAnalysisAPI(config=context.obj)


mutant.add_command(resolve_compression)
mutant.add_command(link)
mutant.add_command(store)
mutant.add_command(store_available)


@mutant.command("config-case")
@DRY_RUN
@ARGUMENT_CASE_ID
@click.pass_obj
def config_case(context: CGConfig, dry_run: bool, case_id: str) -> None:
    """Create config file for a case"""
    analysis_api: MutantAnalysisAPI = context.meta_apis["analysis_api"]
    analysis_api.create_case_config(case_id=case_id, dry_run=dry_run)


@mutant.command("run")
@DRY_RUN
@ARGUMENT_CASE_ID
@click.pass_obj
def run(context: CGConfig, dry_run: bool, case_id: str, config_artic: str = None) -> None:
    """Run mutant analysis command for a case"""
    analysis_api: MutantAnalysisAPI = context.meta_apis["analysis_api"]
    analysis_api.check_analysis_ongoing(case_id=case_id)
    if not dry_run:
        analysis_api.add_pending_trailblazer_analysis(case_id=case_id)
        analysis_api.set_statusdb_action(case_id=case_id, action="running")
    try:
        analysis_api.run_analysis(case_id=case_id, dry_run=dry_run, config_artic=config_artic)
    except:
        analysis_api.set_statusdb_action(case_id=case_id, action=None)
        raise


@mutant.command("start")
@DRY_RUN
@ARGUMENT_CASE_ID
@OPTION_ANALYSIS_PARAMETERS_CONFIG
@click.pass_context
def start(context: click.Context, dry_run: bool, case_id: str, config_artic: str) -> None:
    """Start full analysis workflow for a case"""
    analysis_api: MutantAnalysisAPI = context.obj.meta_apis["analysis_api"]
    analysis_api.prepare_fastq_files(case_id=case_id, dry_run=dry_run)
    context.invoke(link, case_id=case_id, dry_run=dry_run)
    context.invoke(config_case, case_id=case_id, dry_run=dry_run)
    context.invoke(run, case_id=case_id, dry_run=dry_run, config_artic=config_artic)
    context.invoke(store, case_id=case_id, dry_run=dry_run)


@mutant.command("start-available")
@DRY_RUN
@click.pass_context
def start_available(context: click.Context, dry_run: bool = False):
    """Start full analysis workflow for all cases ready for analysis"""

    analysis_api: MutantAnalysisAPI = context.obj.meta_apis["analysis_api"]

    exit_code: int = EXIT_SUCCESS
    for case in analysis_api.get_cases_ready_for_analysis():
        try:
            context.invoke(start, case_id=case.internal_id, dry_run=dry_run)
        except AnalysisNotReadyError as error:
            LOG.error(error)
        except CgError as error:
            LOG.error(error)
            exit_code = EXIT_FAIL
        except Exception as error:
            LOG.error(f"Unspecified error occurred: {error}")
            exit_code = EXIT_FAIL
    if exit_code:
        raise click.Abort
